// #![cfg_attr(debug_assertions, allow(dead_code, unused))]

use fltk::{
    app::App, button::*, enums::*, frame::Frame, group::{Flex, Tabs}, image::*, prelude::*, terminal::{self, RedrawStyle}, window::{DoubleWindow, Window}
};

use fltk_theme::{
    /*color_themes, widget_themes, ColorTheme, SchemeType,*/ ThemeType, /*WidgetScheme,*/ WidgetTheme,
};

use std::{thread, time};
use std::process::Command;
use std::env;
use std::fs::File;
use std::io::{self, BufRead};
use std::path::Path;

use chrono::Datelike;

fn read_lines<P>(filename: P) -> io::Result<io::Lines<io::BufReader<File>>>
where P: AsRef<Path>, {
    let file = File::open(filename)?;
    Ok(io::BufReader::new(file).lines())
}

fn run_cmd(cmd_string: &str, lang_env_var: Option<&str>) {
    if lang_env_var.is_some() {
        let key: &str = "LANG";
        env::set_var(key, lang_env_var.unwrap_or("C.UTF-8"));
    }

    let args_vec: Vec<&str> = cmd_string.split_whitespace().collect();

    if args_vec.len() == 1 {
        let cmd = args_vec[0];

        let status = Command::new(cmd).output();
        println!("{:#?}", status);
    } else {
        let cmd = args_vec[0];
        let new_args_vec = &args_vec[1..].to_vec();

        let status = Command::new(cmd).args(new_args_vec).output();
        println!("{:#?}", status);
    }
}

fn main() {
    let current_date = chrono::Utc::now();
    let current_year = format!("{}", current_date.year());

    let width = 800;
    let height = 700;

    let application = App::default();

    // let application = App::default().with_scheme(app::Scheme::Gtk); // Base, Gleam, Gtk, Oxy, Plastic

    // let widget_theme = WidgetTheme::new(ThemeType::Dark);
    // widget_theme.apply();

    let widget_theme = WidgetTheme::new(ThemeType::Metro);
    widget_theme.apply();

    // let widget_scheme = WidgetScheme::new(SchemeType::Fluent);
    // widget_scheme.apply();

    // let color_theme = ColorTheme::new(color_themes::GRAY_THEME);
    // color_theme.apply();

    let mut win: DoubleWindow = Window::default()
        .with_size(width, height)
        .center_screen()
        .with_label("Melawy Linux: Welcome");

    let icon =
        SvgImage::load("/usr/share/melawy-linux-welcome/assets/icons/Melawy.svg").unwrap();
    win.set_icon(Some(icon));

    let mut scale_image =
        SvgImage::load("/usr/share/melawy-linux-welcome/assets/icons/melawy-linux.svg").unwrap();

    //  -----------------------------------------------------------

    let mut tabs = Tabs::default_fill();

    let mut grp_install_new_sys = Flex::default_fill()
        .with_label("Main\t\t")
        .column();


    let mut frame1 = Frame::default().with_label("Empty Data");
    frame1.set_frame(FrameType::BorderBox);

    scale_image.scale(500, 500, true, true);
    frame1.set_image(Some(scale_image));
    frame1.set_label("");


    let mut col_install_new_sys = Flex::default().column();
    col_install_new_sys.set_margin(10);
    col_install_new_sys.set_pad(10);

    let mut btn_install_system = Button::default().with_label("INSTALL NEW SYSTEM");


    btn_install_system.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/lib/kf6/kdesu dbus-launch calamares";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    col_install_new_sys.end();
    grp_install_new_sys.fixed(&col_install_new_sys, 100);
    grp_install_new_sys.end();

    //  -----------------------------------------------------------

    let grp_sys_and_repo = Flex::default_fill()
        .with_label("System and repositories\t\t")
        .row();
    let mut col_sys_and_repo = Flex::default().column();
    col_sys_and_repo.set_margin(10);
    col_sys_and_repo.set_pad(10);

    let mut btn_update_repo_mirrorlists = Button::default().with_label("Update repo mirrorlists");

    btn_update_repo_mirrorlists.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e /bin/sh -c /usr/share/melawy-linux-welcome/scripts/update_mirrorlist.sh";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_fix_repo_keys = Button::default().with_label("Fix repo keys");

    btn_fix_repo_keys.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e /bin/sh -c /usr/share/melawy-linux-welcome/scripts/key_sign.sh";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_update_system = Button::default().with_label("Update system");

    btn_update_system.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "sudo arch-linux-updater";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_unlock_repo_databases = Button::default().with_label("Unlock repo databases");

    btn_unlock_repo_databases.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e /bin/sh -c /usr/share/melawy-linux-welcome/scripts/unlock_pacman_db.sh";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_clear_old_package_cache = Button::default().with_label("Clear old package cache");

    btn_clear_old_package_cache.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e sudo pacman -Sc";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_clear_all_package_cache = Button::default().with_label("Clear all package cache");

    btn_clear_all_package_cache.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e sudo pacman -Scc";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_check_missing_files_in_packages =
        Button::default().with_label("Fix missing files in packages");

    btn_check_missing_files_in_packages.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e /bin/sh -c /usr/share/melawy-linux-welcome/scripts/fix_missing_files_in_packages.sh";
                let _ = run_cmd(cmd, Some("C.UTF-8"));

                b.activate();
            }
        });
    });

    let mut btn_reinstall_all_packages = Button::default().with_label("Reinstall all packages");

    btn_reinstall_all_packages.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "/usr/bin/konsole --noclose --separate -e /bin/sh -c /usr/share/melawy-linux-welcome/scripts/reinstall_all_packages.sh";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    col_sys_and_repo.end();
    grp_sys_and_repo.end();

    //  -----------------------------------------------------------

    let grp_services = Flex::default_fill().with_label("Services\t\t").row();
    let mut col_services = Flex::default().column();
    col_services.set_margin(10);
    col_services.set_pad(10);

    let mut btn_restart_kde_plasma = Button::default().with_label("Restart KDE Plasma");

    btn_restart_kde_plasma.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "systemctl --user restart plasma-plasmashell";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_restart_kde_effects = Button::default().with_label("Restart KDE Plasma Effects");

    btn_restart_kde_effects.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "qdbus org.kde.KWin /KWin reconfigure";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_reset_kde_plasma_layout = Button::default().with_label("Reset KDE Plasma Layout");

    btn_reset_kde_plasma_layout.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let theme = "com.github.Melawy.Melawy-round-gray.Nier-A2.desktop";
                let cmd = format!("plasma-apply-lookandfeel -a {theme} --resetLayout");
                let _ = run_cmd(cmd.as_str(), None);

                let time_to_sleep = time::Duration::from_secs(5);
                thread::sleep(time_to_sleep);

                let cmd = "systemctl --user restart plasma-plasmashell";
                let _ = run_cmd(cmd, None);

                let time_to_sleep = time::Duration::from_secs(5);
                thread::sleep(time_to_sleep);

                let cmd = "klassy-settings -w Melawy_3";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_restart_tuned =
        Button::default().with_label("Restart Tuned and Tuned - Power Profile Daemon");

    btn_restart_tuned.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "systemctl restart tuned.service tuned-ppd.service";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_restart_dnscrypt_proxy = Button::default().with_label("Restart DNS Crypt Proxy");

    btn_restart_dnscrypt_proxy.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "systemctl restart dnscrypt-proxy.service";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_restart_tor = Button::default().with_label("Restart Tor");

    btn_restart_tor.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "systemctl restart tor.service";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    col_services.end();
    grp_services.end();

    //  -----------------------------------------------------------

    let grp_information = Flex::default_fill().with_label("Information\t\t").row();
    let mut col_information = Flex::default().column();
    col_information.set_margin(10);
    col_information.set_pad(10);

    let mut btn_visit_site = Button::default().with_label("Visit Melawy Linux site");

    btn_visit_site.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://melawy.ru";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let row_help = Flex::default().row();

    let mut btn_get_help = Button::default().with_label("I need help");

    btn_get_help.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://melawy.ru/help";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_give_help = Button::default().with_label("I'm ready to help");

    btn_give_help.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://melawy.ru/help";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_donate = Button::default().with_label("Donate");

    btn_donate.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://melawy.ru/donate";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    row_help.end();


    let row_telegram = Flex::default().row();

    let mut btn_telegram_channel = Button::default().with_label("Telegram Channel");

    btn_telegram_channel.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://t.me/Melawy_Linux";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_telegram_chat = Button::default().with_label("Telegram Chat");

    btn_telegram_chat.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://t.me/Melawy_Linux_Chat";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    row_telegram.end();

    let row_other_media = Flex::default().row();

    let mut btn_youtube = Button::default().with_label("YouTube");

    btn_youtube.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://www.youtube.com/@Melawy";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_discord = Button::default().with_label("Discrord");

    btn_discord.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://discord.gg/725zXx7RhJ";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    let mut btn_mastodon = Button::default().with_label("Mastodon");

    btn_mastodon.set_callback(move |b| {
        b.deactivate();
        std::thread::spawn({
            let mut b = b.clone();
            move || {
                let cmd = "xdg-open https://techhub.social/@Melawy";
                let _ = run_cmd(cmd, None);

                b.activate();
            }
        });
    });

    row_other_media.end();

    col_information.end();
    grp_information.end();

    //  -----------------------------------------------------------

    let grp_about = Flex::default_fill()
        .with_label("About\t\t")
        .column();

    let mut tabs_mini_about = Tabs::default_fill();

    let grp_mini_about_en = Flex::default_fill()
        .with_label("EN\t\t")
        .column();

    let mut term_en = terminal::Terminal::default_fill();
    term_en.set_color(Color::Light1);
    term_en.set_text_color(Color::Gray0);
    term_en.set_redraw_style(RedrawStyle::PerWrite);

    if let Ok(lines) = read_lines("/usr/share/melawy-linux-welcome/assets/text/about_en.txt") {
        // Получает итератор, который возвращает Option
        for line in lines {
            if let Ok(text_string) = line {
                term_en.append(format!("{}\n", text_string).as_str());
            }
        }
    }

    let copyright_en = format!("\n(с) Valeria Fadeeva, 2023 - {}\n", current_year.clone());

    term_en.append(copyright_en.as_str());

    grp_mini_about_en.end();

    //  -----------------------------------------------------------

    let grp_mini_about_ru = Flex::default_fill()
        .with_label("RU\t\t")
        .column();

    let mut term_ru = terminal::Terminal::default_fill();
    term_ru.set_color(Color::Light1);
    term_ru.set_text_color(Color::Gray0);
    term_ru.set_redraw_style(RedrawStyle::PerWrite);

    if let Ok(lines) = read_lines("/usr/share/melawy-linux-welcome/assets/text/about_ru.txt") {
        // Получает итератор, который возвращает Option
        for line in lines {
            if let Ok(text_string) = line {
                term_ru.append(format!("{}\n", text_string).as_str());
            }
        }
    }

    let copyright_ru = format!("\n(с) Валерия Фадеева, 2023 - {}\n", current_year.clone());

    term_ru.append(copyright_ru.as_str());

    grp_mini_about_ru.end();

    tabs_mini_about.end();
    tabs_mini_about.auto_layout();

    grp_about.end();

    //  -----------------------------------------------------------

    tabs.end();
    tabs.auto_layout();

    //  -----------------------------------------------------------

    win.make_resizable(true);
    win.end();
    win.show();

    application.run().unwrap();
}
